@@ -20,6 +20,7 @@ import os
 import argparse
 
 from miauth.mi.miclient import MiClient
+from miauth.mi.m365client import M365Client
 from miauth.nb.nbclient import NbClient
 from miauth.nb.nbcrypto import NbCrypto
 
@@ -27,38 +28,42 @@ from miauth.ble.blue import BluePy
 
 parser = argparse.ArgumentParser()
 parser.add_argument("mac", help="mac address of target device")
+parser.add_argument("-m", "--m365", action='store_true', help="use M365 protocol instead")
 parser.add_argument("-n", "--nb", action='store_true', help="use Nb protocol instead")
 parser.add_argument("-c", "--command", help="send command (w/o checksum) to uart and print reply")
 parser.add_argument("-s", "--serial", action='store_true', help="retrieve serial number")
-parser.add_argument("-v", "--version", action='store_true', help="retrieve firmware version")
-parser.add_argument("-d", "--debug", action='store_true', help="activate debug log")
+parser.add_argument("-f", "--fwver", action='store_true', help="retrieve firmware version")
+parser.add_argument("-v", "--verbose", action='store_true', help="activate verbose (debug) log")
 
 parser.add_argument("-r", "--register", action='store_true',
                     help="register with device / create token (caution: will lose bond to all other apps)")
+parser.add_argument("-d", "--register_did", help="supply did for registeration (advanced)")
 parser.add_argument("-t", "--token_file", default="./mi_token",
                     help="path to mi token file (default: ./mi_token)")
 
 args = parser.parse_args()
+print(args)
 
 
 def nb_main(ble):
-    nc = NbClient(ble, NbCrypto(), debug=args.debug)
+    nc = NbClient(ble, NbCrypto(), debug=args.verbose)
 
     print("Connecting")
     nc.connect()
+
     print("Authenticating")
     nc.auth()
 
     if args.command:
         resp = nc.comm(args.command)
-        print("UART reply:", resp.hex())
+        print("UART reply:", resp.hex(" "))
 
     # NOTE: don't send checksums!
     if args.serial:
         print("Retrieving serial number")
         resp = nc.comm("5aa5013d2001100e")
         print("Serial no.:", resp.decode())
-    if args.version:
+    if args.fwver:
         print("Retrieving firmware version")
         resp = nc.comm("5aa5013d20011a10")
         print("Firmware version:", f"{resp[0]}.{resp[1]}")
@@ -67,14 +72,40 @@ def nb_main(ble):
     nc.disconnect()
 
 
-def mi_main(ble):
-    mc = MiClient(ble, debug=args.debug)
+def m365_main(ble):
+    mc = M365Client(ble, debug=args.verbose)
+
     print("Connecting")
-    ble.connect()
+    mc.connect()
+    mc.recover_key()
+
+    if args.command:
+        resp = mc.comm(args.command)
+        print("UART reply:", resp.hex(" "))
+
+    # NOTE: don't send checksums!
+    if args.serial:
+        print("Retrieving serial number")
+        resp = mc.comm("55aa032001100e")
+        print("Serial no.:", resp.decode())
+    if args.fwver:
+        print("Retrieving firmware version")
+        resp = mc.comm("55aa0320011a10")
+        print("Firmware version:", f"{resp[0]}.{resp[1]}")
+
+    print("Disconnecting")
+    mc.disconnect()
+
+
+def mi_main(ble):
+    mc = MiClient(ble, debug=args.verbose)
+
+    print("Connecting")
+    mc.connect()
 
     if args.register:
         print("Registering")
-        mc.register()
+        mc.register(did=args.register_did)
         mc.save_token(args.token_file)
         print("Saved token to:", args.token_file)
 
@@ -94,20 +125,20 @@ Caution: After registration this device will lose coupling to all other apps (re
     if args.command:
         print("Sending command:", args.command)
         resp = mc.comm(args.command)
-        print("UART reply:", resp.hex())
+        print("UART reply:", resp.hex(" "))
 
     # NOTE: don't send checksum
     if args.serial:
         print("Retrieving serial number")
         resp = mc.comm("55aa032001100e")
         print("Serial number:", resp.decode())
-    if args.version:
+    if args.fwver:
         print("Retrieving firmware version")
         resp = mc.comm("55aa0320011a10")
         print("Firmware version:", f"{resp[0]}.{resp[1]}")
 
     print("Disconnecting")
-    ble.disconnect()
+    mc.disconnect()
 
 
 def main():
@@ -117,6 +148,8 @@ def main():
     if args.nb:
         print("Using Nb")
         nb_main(ble)
+    elif args.m365:
+        m365_main(ble)
     else:
         print("Using Mi")
         mi_main(ble)
