@@ -16,15 +16,17 @@
 #     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 import time
+from enum import Enum
 
 from miauth.mi.micommand import MiCommand
 from miauth.mi.micrypto import MiCrypto
 from miauth.ble.base import BLEBase
 from miauth.ble.uuid import UUID
+from miauth.util import crc16
 
 
 class MiClient(object):
-    class State(object):
+    class State(Enum):
         INIT = -1
         RECV_INFO = 0
         SEND_KEY = 1
@@ -64,6 +66,38 @@ class MiClient(object):
         # counter for sent uart commands
         self.uart_it = 0
 
+    def main_handler(self, data):
+        if not data:
+            if self.debug:
+                print("<- Empty data")
+            return
+
+        frm = data[0] + 0x100 * data[1]
+        if self.debug:
+            print("<-", data.hex())
+
+        frm = data[0]
+        if len(data) > 1:
+            frm += 0x100 * data[1]
+
+        if self.get_state() in [MiClient.State.RECV_INFO,
+                                MiClient.State.RECV_KEY]:
+            self.receive_handler(frm, data)
+        elif self.get_state() in [MiClient.State.SEND_KEY,
+                                  MiClient.State.SEND_DID]:
+            self.send_handler(frm, data)
+        elif self.get_state() == MiClient.State.CONFIRM:
+            self.confirm_handler(frm)
+        elif self.get_state() == MiClient.State.COMM:
+            # TODO: check if correct number of frames received
+            self.received_data += data
+
+    def connect(self):
+        self.ble.connect()
+
+    def disconnect(self):
+        self.ble.disconnect()
+
     def reset(self):
         self.ble.disconnect()
         self.__init__(self.ble, debug=self.debug)
@@ -82,26 +116,6 @@ class MiClient(object):
         if f is not None:
             f()
 
-    def main_handler(self, data):
-        if self.debug:
-            print("<-", data.hex(), self.get_state())
-
-        frm = data[0]
-        if len(data) > 1:
-            frm += 0x100 * data[1]
-
-        if self.get_state() in [MiClient.State.RECV_INFO,
-                                MiClient.State.RECV_KEY]:
-            self.receive_handler(frm, data)
-        elif self.get_state() in [MiClient.State.SEND_KEY,
-                                  MiClient.State.SEND_DID]:
-            self.send_handler(frm, data)
-        elif self.get_state() == MiClient.State.CONFIRM:
-            self.confirm_handler(frm)
-        elif self.get_state() == MiClient.State.COMM:
-            # TODO: check if correct number of frames received
-            self.received_data += data
-
     def receive_handler(self, frm, data):
         if frm == 0:
             self.receive_frames = data[4] + 0x100 * data[5]
@@ -121,17 +135,17 @@ class MiClient(object):
 
     def send_handler(self, frm, data):
         if frm != 0:
-            return
+            raise Exception("Mi unknown error. Try register.")
 
         if data == MiCommand.RCV_RDY:
             if self.debug:
                 print("Mi ready to receive key")
             self.ble.write_parcel(UUID.AVDTP, self.send_data)
-        if data == MiCommand.RCV_TOUT:
+        elif data == MiCommand.RCV_TOUT:
             raise Exception("Mi sent RCV timeout.")
-        if data == MiCommand.RCV_ERR:
+        elif data == MiCommand.RCV_ERR:
             raise Exception("Mi sent some RCV error?")
-        if data == MiCommand.RCV_OK:
+        elif data == MiCommand.RCV_OK:
             if self.debug:
                 print("Mi confirmed key receive")
             self.next_state()
@@ -159,13 +173,15 @@ class MiClient(object):
         bind_key = derived_key[12:28]
         a = derived_key[28:44]
 
-        did_ct = MiCrypto.encrypt_did(a, self.remote_info[4:])
+        did = self.remote_info
+        did_ct = MiCrypto.encrypt_did(a, did)
 
         if self.debug:
             print("eShareKey:", e_share_key.hex())
             print("HKDF result: ", derived_key.hex())
             print("token:", token.hex())
             print("bind_key:", bind_key.hex())
+            print("did:", did.decode())
             print("A:", a.hex())
             print("AES did CT: ", did_ct.hex())
 
@@ -192,7 +208,7 @@ class MiClient(object):
 
         return info, expected_remote_info, keys
 
-    def register(self):
+    def register(self, did=None):
         priv_key, pub_key = MiCrypto.gen_keypair()
         if self.debug:
             print("Private Key (Val):", MiCrypto.private_key_to_val(priv_key))
@@ -202,7 +218,18 @@ class MiClient(object):
             self.ble.write(UUID.UPNP, MiCommand.CMD_GET_INFO)
 
         def on_send_key_state():
-            self.remote_info = self.received_data
+            self.remote_info = self.received_data[4:]
+            if not self.remote_info:
+                if did is None:
+                    raise Exception("Remote info empty, "
+                                    "connect device to official app first "
+                                    "or supply 'register_did' parameter.")
+                self.remote_info = did.encode() + b'\0'
+            if len(self.remote_info) != 20:
+                raise Exception("Remote info has wrong length.")
+
+            if self.debug:
+                print("Remote info received:", self.remote_info.hex())
 
             self.send_data = MiCrypto.pub_key_to_bytes(pub_key)
             self.ble.write(UUID.UPNP, MiCommand.CMD_SET_KEY)
@@ -210,6 +237,8 @@ class MiClient(object):
 
         def on_send_did_state():
             self.remote_key = self.received_data
+            if self.debug:
+                print("Remote key received:", self.remote_key.hex())
 
             self.send_data, self.token = self.calc_did(priv_key)
             self.ble.write(UUID.AVDTP, MiCommand.CMD_SEND_DID)
@@ -241,7 +270,7 @@ class MiClient(object):
                 time.sleep(5)
                 self.ble.connect()
 
-                return self.register()  # return because of recursion
+                return self.register(did=did)  # return because of recursion
             else:
                 break
 
@@ -263,13 +292,17 @@ class MiClient(object):
 
         def on_recv_info_state():
             self.remote_key = self.received_data
+            if self.debug:
+                print("Remote key received:", self.remote_key.hex())
 
         def on_send_did_state():
             self.remote_info = self.received_data
+            if self.debug:
+                print("Remote info received:", self.remote_info.hex())
 
             self.send_data, expected_remote_info, self.keys = self.calc_login_info(rand_key)
             assert self.remote_info == expected_remote_info, \
-                f"{self.remote_info.hex()} != {expected_remote_info.hex()}"
+                f"{self.remote_info.hex(' ')} != {expected_remote_info.hex(' ')}"
             self.ble.write(UUID.AVDTP, MiCommand.CMD_SEND_INFO)
 
         self.seq = (
@@ -316,15 +349,15 @@ class MiClient(object):
                                     self.keys['app_iv'],
                                     cmd,
                                     it=self.uart_it)
+
         self.ble.write_chunked(UUID.TX, res)
         self.uart_it += 1
 
         self.ble.wait_notify()
 
         if not self.received_data:
-            #raise Exception("No answer received. Firmware not supported.")
             print("No answer received")
-            return
+            return bytes()
 
         return MiCrypto.decrypt_uart(
             self.keys['dev_key'],
